***** CALCULATIONS OF TARGETS FOR 2015-2020 QUALITY INCENTIVE SCHEME *****

******************** MODIFIED FOR COMPARING WITH DPP3 ********************

*The purpose of this do-file is to convert the individual interruption data provided by EDBs into
*daily and annual non-normalised and normalised SAIFI and SAIDI values in accordance to possible ComCom criteria
*This is applicable for for years 2015-2020, namely for the new quality incentive scheme 

set more off

cd

foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "Electricity Ashburton" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {

	use "`c(pwd)'\Inputs\DPP2 data.dta", clear
	keep if class=="C"  // only use unplanned interruptions
	keep if edb=="`edb'"
	
	*keep if edb=="Nelson Electricity"
/*
*Generating values for SAIDI and SAIFI calculated using customer minutes, customers affected and total ICPs		
	gen saidi_indiv = minutes/icps
	gen saifi_indiv = customers/icps

*Decision made to give planned interruptions given a 50% weighting
	replace saidi_indiv = minutes/icps*0.5 if class=="B"
	replace saifi_indiv = customers/icps*0.5 if class=="B"
		
save "`edb' CC indiv", replace

*/		
*********************************
****   INDIVIDUAL TO DAILY   ****
*********************************

*Aggregating individual interruptions to daily values using CC calcs
	egen saidi_daily = total(saidi_indiv), by(edb date class)
	egen saifi_daily = total(saifi_indiv), by(edb date class)

*Only want one daily SAIDI/SAIFI planned/unplanned for each day for each EDB
	bysort edb date class: gen unique = _n==1
	drop if unique==0
	
*Now have the data required for daily SAIDI and SAIFI
*Keep only required and vars for next step. Keep first observation for each day
	keep edb disc_yr date saidi_daily saifi_daily class
	drop if saidi_daily==. & saifi_daily==.
	
*For a complete dataset we want all dates between 1 April 2004 and 31 March 2013 included
*I have pre programmed a dates dta-file with all these dates and can be merged
	merge m:1 date using "`c(pwd)'\Inputs\dates"
	drop if date > date("31032014","DMY")
	replace edb=edb[_n-1] if edb==""
	sort date

	
**********************************************
***   CALCULATION OF BOUNDARY VALUES   ***
**********************************************

*The SAIDI and SAIFI boundary values is the 23rd highest SAIDI and SAIFI day respectively over the 10 year reference years
*The new reference years for the reset are the 2005-2014 disclosure years.
*Note that if a shorter (or longer) reference period is used a different day applies (based on 2.3 MEDs per year)
	
	egen rank_saidi = rank(-saidi_daily ) if class=="C", track
	egen rank_saifi = rank(-saifi_daily ) if class=="C", track
	gen temp_saidi= saidi_daily if rank_saidi==23 & (edb!="Electricity Invercargill" | edb!="Nelson Electricity")
		replace temp_saidi = saidi_daily if rank_saidi==15 & edb=="Electricity Invercargill" // small EDB adjustment
		replace temp_saidi = saidi_daily if rank_saidi==7 & edb=="Nelson Electricity" // small EDB adjustment
	
	gen temp_saifi= saifi_daily if rank_saifi==23 & (edb!="Electricity Invercargill" | edb!="Nelson Electricity")
		replace temp_saifi = saifi_daily if rank_saifi==15 & edb=="Electricity Invercargill" // small EDB adjustment
		replace temp_saifi = saifi_daily if rank_saifi==7 & edb=="Nelson Electricity" // small EDB adjustment
	
	egen bound_saidi = mean(temp_saidi)
	egen bound_saifi = mean(temp_saifi)	

*Normalise the daily reliability measures
*Replace daily SAIDI/SAIFI with boundary values if a daily value exceeds the boundary value
	gen Nsaidi = saidi_daily
	gen Nsaifi = saifi_daily
	
*Decision for the final is that the trigger for a major event day will be decoupled, therefore
*SAIDI MED triggered where SAIDI boundary exceeded and SAIFI MED triggered where SAIFI boundary exceeded.
*Decision for the final is to replace major event days with the respective SAIDI/SAIFI boundary values
	replace Nsaidi = bound_saidi if Nsaidi>bound_saidi & class=="C"
	replace Nsaifi = bound_saifi if Nsaifi>bound_saifi & class=="C"

*Now that unplanned normalised can bring the SAIDI and SAIFI days together
	egen saidi_Ndaily = total(Nsaidi) if Nsaidi!=., by(date)
	egen saifi_Ndaily = total(Nsaifi) if Nsaifi!=., by(date)
	bysort date: gen unique = _n==1
	drop if unique==0
	keep edb disc_yr date saidi_Ndaily saifi_Ndaily bound_saidi bound_saifi 

*Save these dailies for each EDB
save "`c(pwd)'\Interim\\`edb' CC daily", replace
	
	
*********************************************
***   DAILY TO TARGETS CAPS AND COLLARS   ***
*********************************************

*Use daily data to calculate targets, caps and collars for SAIDI/SAIFI
*Need to replace zero-days with 0 for calculation of std dev
	replace saidi_Ndaily=0 if saidi_Ndaily==.
	replace saifi_Ndaily=0 if saifi_Ndaily==.
	
*The target is the annual average of the normalised dataset reference period (10 years) 
	egen saidi_target = total(saidi_Ndaily)
	replace saidi_target = saidi_target/10
	egen saifi_target = total(saifi_Ndaily)
	replace saifi_target = saifi_target/10
	
*Calculation of the daily standard deviation and converted to an annual variance
	egen Osaidi = sd(saidi_Ndaily)
	replace Osaidi = Osaidi*(365)^(1/2)
	egen Osaifi = sd(saifi_Ndaily)
	replace Osaifi = Osaifi*(365)^(1/2)
	
*Calculation of the annuals for saving
	egen saidi_Nannual = total(saidi_Ndaily), by(disc_yr)
	egen saifi_Nannual = total(saifi_Ndaily), by(disc_yr)
	bysort edb disc_yr: gen unique = _n==1
	drop if unique==0
	drop unique
save "`c(pwd)'\Interim\\`edb' CC annual", replace
	
*Calculation of caps and collars
	bysort edb: gen unique = _n==1
	drop if unique==0
	
*Cap is target plus std dev variance: Collar is target minus std dev variance
	gen saidi_cap = saidi_target + Osaidi * 1.5 // change to annual 1.5 sd in DPP3
	gen saifi_cap = saifi_target + Osaifi * 1.5 // change to annual 1.5 sd in DPP3
	gen saidi_collar = saidi_target - Osaidi
	gen saifi_collar = saifi_target - Osaifi
	
*Keep relevant variables - boundary, target, cap, collar
	keep edb bound_saidi bound_saifi saidi_target saifi_target saidi_cap saifi_cap saidi_collar saifi_collar Osaidi Osaifi
	order edb bound_saidi Osaidi saidi_collar saidi_target saidi_cap bound_saifi Osaifi saifi_collar saifi_target saifi_cap

save "`c(pwd)'\Interim\\`edb' CC targets", replace
}


*Appending all the targets, caps, collars, boundaries and std devs, along with an excel version
clear
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "Electricity Ashburton" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {

	append using "`c(pwd)'\Interim\\`edb' CC targets"
}	

replace edb = "EA Networks" if edb=="Electricity Ashburton"
sort edb
keep  edb bound_saidi saidi_target saidi_cap bound_saifi saifi_target saifi_cap
save "`c(pwd)'\Outputs\DPP2 unplanned counterfactual", replace
*export excel using "EDB Quality Data.xlsx", sheetreplace firstrow(variables) sheet(DPP2_alt_targets)

*Removing temporary edb specific daily and annual values, now that merged into one dataset
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "Electricity Ashburton" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	*erase "`edb' CC indiv.dta"
	erase "`c(pwd)'\Interim\\`edb' CC daily.dta"
	erase "`c(pwd)'\Interim\\`edb' CC annual.dta"
	erase "`c(pwd)'\Interim\\`edb' CC targets.dta"
}
*end


********************* DPP3 UPDATE USING METHOD ABOVE *********************

foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {

	use "`c(pwd)'\Inputs\EDB interruptions", clear
	
	gen date = dofc(Start_Date_Time)
	rename (Disclosure_Year Planned_Unplanned SAIDI SAIFI) (disc_yr class saidi_indiv saifi_indiv)
	keep edb disc_yr class saidi_indiv saifi_indiv date
	format date %td
	
	keep if class=="Unplanned"  // only use unplanned interruptions
	keep if edb=="`edb'"
	
	*keep if edb=="Nelson Electricity"
/*
*Generating values for SAIDI and SAIFI calculated using customer minutes, customers affected and total ICPs		
	gen saidi_indiv = minutes/icps
	gen saifi_indiv = customers/icps

*Decision made to give planned interruptions given a 50% weighting
	replace saidi_indiv = minutes/icps*0.5 if class=="B"
	replace saifi_indiv = customers/icps*0.5 if class=="B"
		
save "`edb' CC indiv", replace

*/		
*********************************
****   INDIVIDUAL TO DAILY   ****
*********************************

*Aggregating individual interruptions to daily values using CC calcs
	egen saidi_daily = total(saidi_indiv), by(edb date class)
	egen saifi_daily = total(saifi_indiv), by(edb date class)

*Only want one daily SAIDI/SAIFI planned/unplanned for each day for each EDB
	bysort edb date class: gen unique = _n==1
	drop if unique==0
	
*Now have the data required for daily SAIDI and SAIFI
*Keep only required and vars for next step. Keep first observation for each day
	keep edb disc_yr date saidi_daily saifi_daily class
	drop if saidi_daily==. & saifi_daily==.
	
*For a complete dataset we want all dates between 1 April 2004 and 31 March 2013 included
*I have pre programmed a dates dta-file with all these dates and can be merged
	merge m:1 date using "`c(pwd)'\Inputs\dates"
	drop if date < date("01042008","DMY") | date > date("31032018","DMY") 
	replace edb=edb[_n-1] if edb==""
	sort date

	
**********************************************
***   CALCULATION OF BOUNDARY VALUES   ***
**********************************************

*The SAIDI and SAIFI boundary values is the 23rd highest SAIDI and SAIFI day respectively over the 10 year reference years
*The new reference years for the reset are the 2005-2014 disclosure years.
*Note that if a shorter (or longer) reference period is used a different day applies (based on 2.3 MEDs per year)
	
*Using boundary values derived from above 2004-14 dataset for consistency
	drop _merge
	merge m:1 edb using "S:\16504 EDB DPP reset 2020\2. Draft publication\Reliability\DPP2 v DPP3 testing\DPP2 (modified)\output.dta", keepusing(bound_saidi bound_saifi)
	drop if edb != "`edb'"
	
*Normalise the daily reliability measures
*Replace daily SAIDI/SAIFI with boundary values if a daily value exceeds the boundary value
	gen Nsaidi = saidi_daily
	gen Nsaifi = saifi_daily
	
*Decision for the final is that the trigger for a major event day will be decoupled, therefore
*SAIDI MED triggered where SAIDI boundary exceeded and SAIFI MED triggered where SAIFI boundary exceeded.
*Decision for the final is to replace major event days with the respective SAIDI/SAIFI boundary values
	replace Nsaidi = bound_saidi if Nsaidi>bound_saidi & class=="Unplanned"
	replace Nsaifi = bound_saifi if Nsaifi>bound_saifi & class=="Unplanned"

*Now that unplanned normalised can bring the SAIDI and SAIFI days together
	egen saidi_Ndaily = total(Nsaidi) if Nsaidi!=., by(date)
	egen saifi_Ndaily = total(Nsaifi) if Nsaifi!=., by(date)
	bysort date: gen unique = _n==1
	drop if unique==0
	keep edb disc_yr date saidi_Ndaily saifi_Ndaily bound_saidi bound_saifi 

*Save these dailies for each EDB
save "`c(pwd)'\Interim\\`edb' CC daily", replace
	
	
*********************************************
***   DAILY TO TARGETS CAPS AND COLLARS   ***
*********************************************

*Use daily data to calculate targets, caps and collars for SAIDI/SAIFI
*Need to replace zero-days with 0 for calculation of std dev
	replace saidi_Ndaily=0 if saidi_Ndaily==.
	replace saifi_Ndaily=0 if saifi_Ndaily==.
	
*The target is the annual average of the normalised dataset reference period (10 years) 
	egen saidi_target = total(saidi_Ndaily)
	replace saidi_target = saidi_target/10
	egen saifi_target = total(saifi_Ndaily)
	replace saifi_target = saifi_target/10
	
*Calculation of the daily standard deviation and converted to an annual variance
	egen Osaidi = sd(saidi_Ndaily)
	replace Osaidi = Osaidi*(365)^(1/2)
	egen Osaifi = sd(saifi_Ndaily)
	replace Osaifi = Osaifi*(365)^(1/2)
	
*Calculation of the annuals for saving
	egen saidi_Nannual = total(saidi_Ndaily), by(disc_yr)
	egen saifi_Nannual = total(saifi_Ndaily), by(disc_yr)
	bysort edb disc_yr: gen unique = _n==1
	drop if unique==0
	drop unique
save "`c(pwd)'\Interim\\`edb' CC annual", replace
	
*Calculation of caps and collars
	bysort edb: gen unique = _n==1
	drop if unique==0
	
*Cap is target plus std dev variance: Collar is target minus std dev variance
	gen saidi_cap = saidi_target + Osaidi * 1.5 // change to annual 1.5 sd in DPP3
	gen saifi_cap = saifi_target + Osaifi * 1.5 // change to annual 1.5 sd in DPP3
	gen saidi_collar = saidi_target - Osaidi
	gen saifi_collar = saifi_target - Osaifi
	
*Keep relevant variables - boundary, target, cap, collar
	keep edb bound_saidi bound_saifi saidi_target saifi_target saidi_cap saifi_cap saidi_collar saifi_collar Osaidi Osaifi
	order edb bound_saidi Osaidi saidi_collar saidi_target saidi_cap bound_saifi Osaifi saifi_collar saifi_target saifi_cap

save "`c(pwd)'\Interim\\`edb' CC targets", replace
}


*Appending all the targets, caps, collars, boundaries and std devs, along with an excel version
clear
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {

	append using "`c(pwd)'\Interim\\`edb' CC targets"
}	

sort edb
keep  edb bound_saidi saidi_target saidi_cap bound_saifi saifi_target saifi_cap
save "`c(pwd)'\Outputs\DPP3 unplanned counterfactual", replace
*export excel using "EDB Quality Data.xlsx", sheetreplace firstrow(variables) sheet(DPP2_alt_targets)

*Removing temporary edb specific daily and annual values, now that merged into one dataset
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" "Electricity Invercargill" "Horizon Energy" ///
	"The Lines Company" "Nelson Electricity" "Network Tasman" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	*erase "`edb' CC indiv.dta"
	erase "`c(pwd)'\Interim\\`edb' CC daily.dta"
	erase "`c(pwd)'\Interim\\`edb' CC annual.dta"
	erase "`c(pwd)'\Interim\\`edb' CC targets.dta"
}

exit
