*** CALCULATIONS OF 2020-2025 RELIABILITY STANDARDS AND INCENTIVE PARAMETERS ***

******************************** DRAFT DECISION ********************************


* This do-file creates the unplanned and planned SAIDI and SAIFI targets, caps, and collars
cd

***** UNPLANNED INTERRUPTIONS *****

***** DEFINING A MAJOR EVENT *****

* First step is to create dataset of every half-hour over reference period
clear
set obs 175296	// 3652days * 48half-hours
gen time = .
recast double time
format %tc time
replace time = clock("01apr2008 00:00:00","DMYhms") in 1
replace time = time[_n-1]+1800000 if _n!=1	// half-hour = 30minutes = 1800seconds = 1800000millisecs
save "`c(pwd)'\Interim\all_halfhours", replace

* Second step is, using the 53ZD dataset, round each interruption DOWN to the nearest half-hour
use "`c(pwd)'\Inputs\EDB interruptions", clear
gen double time_rounded = 1800000 * floor( Start_Date_Time / 1800000)
format %tc time_rounded

* For now, I am only interested in edb, rounded start time, unplanned interruptions, SAIDI, and SAIFI
keep if Planned_Unplanned == "Unplanned"
save "`c(pwd)'\Interim\Unplanned half hours", replace

* Third step - merge with the half-hour dataset
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" ///
	"Electricity Invercargill" "Horizon Energy" "The Lines Company" "Nelson Electricity" "Network Tasman" ///
	"Orion NZ" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	di "`edb'"
	use "`c(pwd)'\Interim\Unplanned half hours", clear
	keep if edb=="`edb'"
	keep edb time_rounded SAIDI SAIFI
	rename time_rounded time
	merge m:1 time using "`c(pwd)'\Interim\all_halfhours"
	replace edb = edb[_n-1] if edb==""
	sort time

* Fourth step - make sure there is only one observation for each half-hour
	gen duplicate = 1 if time==time[_n-1]
	egen saidi_halfhour = total(SAIDI), by(time)
	egen saifi_halfhour = total(SAIFI), by(time)
	drop if duplicate==1
	drop SAIDI SAIFI
	egen temp_number = count(saidi_halfhour) if saidi_halfhour!=0
	egen number = mean(temp_number)

	foreach var in saidi saifi {
* Fifth step - create the rolling 3 (or n) hour total
		gen cum_`var' = sum(`var'_halfhour)
		gen cum_`var'_6 = cum_`var'[_n-6]
		gen `var'_3hr = cum_`var' - cum_`var'_6

* Sixth step - select boundary value (25MEs * 6periods = 150)
		egen rank_`var' = rank(-`var'_3hr), unique
		gen temp_`var' = `var'_3hr if rank_`var'==150
		replace temp_`var' = `var'_3hr if rank_`var'==98 & edb=="Electricity Invercargill" // small EDB adjustment
		replace temp_`var' = `var'_3hr if rank_`var'==45 & edb=="Nelson Electricity" // small EDB adjustment
		egen bound_`var' = mean(temp_`var')
		}

* Seventh step - tidy up, save, and append boundary values
	save "`c(pwd)'\Interim\\`edb'_halfhour", replace
	keep edb bound_saidi bound_saifi number
	keep if _n==1
	save "`c(pwd)'\Interim\\bv_`edb'", replace
	}

clear
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" ///
	"Electricity Invercargill" "Horizon Energy" "The Lines Company" "Nelson Electricity" "Network Tasman" ///
	"Orion NZ" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	append using "`c(pwd)'\\Interim\bv_`edb'"
	erase "`c(pwd)'\Interim\\bv_`edb'.dta"
	}

order edb
sort edb
save "`c(pwd)'\Interim\Boundary values", replace


*** Apply normalisation to unplanned reference period ***

foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" ///
	"Electricity Invercargill" "Horizon Energy" "The Lines Company" "Nelson Electricity" "Network Tasman" ///
	"Orion NZ" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {

	use "`c(pwd)'\Interim\\`edb'_halfhour", clear
	gen year = yofd(dofc(time))
	gen month = month(dofc(time))
	gen disc_yr = year
	replace disc_yr = disc_yr+1 if month>=4 
	
	foreach var in saidi saifi {
		* capture major events (the three hours leading up to SAIDI or SAIFI exceeding boundary value)
		gen `var'_ME1 = 1 if `var'_3hr >= bound_`var' & `var'_3hr!=.
		gen `var'_ME2 = 1 if `var'_ME1[_n+1]==1
		gen `var'_ME3 = 1 if `var'_ME2[_n+1]==1
		gen `var'_ME4 = 1 if `var'_ME3[_n+1]==1
		gen `var'_ME5 = 1 if `var'_ME4[_n+1]==1
		gen `var'_ME6 = 1 if `var'_ME5[_n+1]==1
		egen `var'_ME = rowtotal(`var'_ME1-`var'_ME6)
		
		* replace SAIDI and SAIFI values with pro-rated boundary value
		replace `var'_halfhour = bound_`var' / 48 if `var'_ME!=0

		* create target = annual average
		egen `var'_target = total(`var'_halfhour)
		replace `var'_target = `var'_target / 10
		
		* create cap/limit = annual average plus 1.5 standard deviations
		egen `var'_deviation = sd(`var'_halfhour)
		replace `var'_deviation = `var'_deviation*(365*48)^(1/2)
		gen `var'_cap = `var'_target + 1.5 * `var'_deviation
		
		* create collar = 0
		gen `var'_collar = 0
		
		* for analysis - create annual normalised SAIDI and SAIFI
		egen `var'_annual = total(`var'_halfhour), by(disc_yr)
		}
	
	keep bound_saidi saidi_target saidi_collar saidi_cap saidi_deviation saidi_annual ///
		 bound_saifi saifi_target saifi_collar saifi_cap saifi_deviation saifi_annual ///
		 edb disc_yr
	
	drop if disc_yr==.
	gen unique = 1 if disc_yr!=disc_yr[_n-1]
	keep if unique==1
	save "`c(pwd)'\Interim\\annual_`edb'", replace
	
	keep if _n==1
	drop disc_yr saidi_annual saifi_annual
	erase "`c(pwd)'\Interim\\`edb'_halfhour.dta"
	save "`c(pwd)'\Interim\\unplanned_`edb'", replace
	}

clear
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" ///
	"Electricity Invercargill" "Horizon Energy" "The Lines Company" "Nelson Electricity" "Network Tasman" ///
	"Orion NZ" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	append using "`c(pwd)'\Interim\\unplanned_`edb'"
	erase "`c(pwd)'\Interim\\unplanned_`edb'.dta"
	}

order edb bound_saidi saidi_target saidi_collar saidi_cap saidi_deviation ///
	      bound_saifi saifi_target saifi_collar saifi_cap saifi_deviation 
drop unique
sort edb
save "`c(pwd)'\Outputs\Unplanned parameters", replace
	

***** PLANNED INTERRUPTIONS *****

* Proposed approach for setting caps and limits much simpler
* Caps are are 3x reference period average
* Regulatory period limit 5years x 3x reference period average

use "`c(pwd)'\Inputs\EDB interruptions", clear
keep if Planned_Unplanned=="Planned" & Disclosure_Year>=2009
egen saidi_target = total(SAIDI), by(edb)
egen saifi_target = total(SAIFI), by(edb)

foreach var in saidi saifi {
	replace `var'_target = `var'_target/10
	gen `var'_collar = 0
	gen `var'_cap = 3 * `var'_target
	gen `var'_limit = 5 * `var'_cap
	}

sort edb
gen unique = 1 if edb!=edb[_n-1]
keep if unique==1
keep edb saidi_target saidi_collar saidi_cap saidi_limit saifi_target saifi_collar saifi_cap saifi_limit
order edb saidi_collar saidi_target saidi_cap saidi_limit saifi_collar saifi_target saifi_cap saifi_limit
save "`c(pwd)'\Outputs\Planned parameters", replace
	

*** Merge for other analysis ***

clear
foreach edb in "Alpine Energy" "Aurora Energy" "Centralines" "Eastland Network" "EA Networks" ///
	"Electricity Invercargill" "Horizon Energy" "The Lines Company" "Nelson Electricity" "Network Tasman" ///
	"Orion NZ" "OtagoNet" "Powerco" "Top Energy" "Unison Networks" "Vector Lines" "Wellington Electricity" {
	
	append using "`c(pwd)'\Interim\\annual_`edb'.dta"
	erase "`c(pwd)'\Interim\\annual_`edb'.dta"
	}
save "`c(pwd)'\Interim\Annual unplanned", replace

exit
